#ifndef __GLOBALS__
#define __GLOBALS__

#include <forms.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>

#include "config.h"
#include "main.h"
#include "pointer.h"
#include "popen2.h"
#include "parse.h"
#include "buttons.h"
#include "structures.h"




// popen2.h
extern FILE *writepipe, *readpipe, *config;
extern int pid;



// parse.h
enum
{
   BREAKPOINT_VIRTUAL,
   BREAKPOINT_PHYSICAL,
   BREAKPOINT_LINEAR
};
#define STRING_LENGTH 128
struct breakpoint_struct {
   int      number;
   int      type;
   int      enabled;
   unsigned segment;
   unsigned offset;
};
struct segment_t {
   unsigned segment;
   unsigned descriptor[2];
   int      valid;
};
struct table_t {
   unsigned base;
   unsigned limit;
};
struct cpu_t {
   unsigned		eax;
   unsigned		ebx;
   unsigned		ecx;
   unsigned		edx;
   unsigned		ebp;
   unsigned		esi;
   unsigned		edi;
   unsigned		esp;
   unsigned		eflags;
   unsigned		eip;
   struct segment_t	cs;
   struct segment_t	ss;
   struct segment_t	ds;
   struct segment_t	es;
   struct segment_t	fs;
   struct segment_t	gs;
   struct segment_t	ldtr;
   struct segment_t	tr;
   struct table_t	gdtr;
   struct table_t	idtr;
   unsigned		dr0;
   unsigned		dr1;
   unsigned		dr2;
   unsigned		dr3;
   unsigned		dr4;
   unsigned		dr5;
   unsigned		dr6;
   unsigned		dr7;
   unsigned		tr3;
   unsigned		tr4;
   unsigned		tr5;
   unsigned		tr6;
   unsigned		tr7;
   unsigned		cr0;
   unsigned		cr1;
   unsigned		cr2;
   unsigned		cr3;
   unsigned		cr4;
   int			inhibit;
};
extern struct breakpoint_struct breakpoints[30];
extern struct cpu_t cpu;
extern unsigned   i_virtual_segment;
extern unsigned   i_virtual_offset;
extern unsigned   i_physical_address;
extern unsigned   i_size;
extern char       i_description[STRING_LENGTH];
extern int        p_step;
extern int        bp_type;


// buttons.h
extern FL_OBJECT *c_hislen_text, *c_sabove_text, *c_sbelow_text, *c_steplock_text, *c_nextlock_text;

extern FL_OBJECT *m_linear, *m_physical;
extern FL_OBJECT *quit_button, *hide_button, *show_button, *state_widgets, *refreshall_button, *dummy;
extern FL_OBJECT *dump_button, *add_button, *remove_button, *clearall_button;
extern FL_OBJECT *go_button, *step_button, *stop_button, *next_button;
extern FL_OBJECT *physical_check, *linear_check, *virtual_check;
extern FL_OBJECT *m_size_byte, *m_size_word, *m_size_dword, *m_size_qword;
extern FL_OBJECT *m_type_hex, *m_type_dec, *m_type_uns, *m_type_oct, *m_type_bin;

extern FL_OBJECT *r_eax_text, *r_ebx_text, *r_ecx_text, *r_edx_text, *r_ebp_text, *r_esi_text, *r_edi_text, *r_esp_text;
extern FL_OBJECT *r_eflags_text, *r_eip_text, *r_cs_text, *r_ss_text, *r_ds_text, *r_es_text, *r_fs_text, *r_gs_text;
extern FL_OBJECT *r_ldtr_text, *r_tr_text, *r_gdtr_text, *r_idtr_text;
extern FL_OBJECT *r_dr0_text, *r_dr1_text, *r_dr2_text, *r_dr3_text, *r_dr4_text, *r_dr5_text, *r_dr6_text, *r_dr7_text;
extern FL_OBJECT *r_tr3_text, *r_tr4_text, *r_tr5_text, *r_tr6_text, *r_tr7_text;                                       
extern FL_OBJECT *r_cr0_text, *r_cr1_text, *r_cr2_text, *r_cr3_text, *r_cr4_text;

extern FL_FORM *window, *f_config_folder, *f_structures_folder, *f_memory_folder, *f_breakpoints_folder;
extern FL_FORM *quitConfirm, *f_history_folder;
extern FL_OBJECT *yes, *no, *i_list;
extern FL_OBJECT *i_address_text, *i_offset_text, *i_instruction_text;
extern FL_OBJECT *b_list, *b_address_text;
extern FL_OBJECT *m_address_text, *m_size_text;
extern FL_OBJECT *s_list, *st_list, *f_list;
extern FL_OBJECT *event, *folder, *main_group;

struct memwin_struct {
	FL_FORM		*window;
	FL_OBJECT	*list, *close_button, *refresh_button;
	unsigned	address;
	unsigned	units;
	unsigned	physical;
	char		unit_size;
	char		unit_format;
	struct memwin_struct *prev;
	struct memwin_struct *next;
};
extern struct memwin_struct *mem_windows;

extern int  next_active, bochs_running, update_state, bochs_offline;
extern char mem_type, mem_size, mem_physical;
extern unsigned steplock_time, nextlock_time;



// structures.h
#define WORD_LEN 99
struct value_struct {
        char name[WORD_LEN];
        size_t size;
};
struct mem_struct {
	FL_FORM		*window;
	FL_OBJECT	*start_text, *total_text;
	FL_OBJECT	*close_button, *refresh_button, *table;
	char		name[WORD_LEN];
	unsigned	base;
	size_t		element_count;
	size_t		value_count;
	struct value_struct *values;
};
extern struct mem_struct *memlist;
extern unsigned stack_above, stack_below;
extern unsigned history_length;
extern int nr_strucs;


#endif
